@@ -1,3 +1,5 @@
+from __future__ import unicode_literals
+
 import re
 
 import pytest
@@ -119,7 +121,7 @@ def test_email_link(data, parse_email, expected):
     assert linkify(data, parse_email=parse_email) == expected
 
 
-@pytest.mark.parametrize('data,expected', [
+@pytest.mark.parametrize('data, expected', [
     (
         '"james"@example.com',
         '''<a href='mailto:"james"@example.com'>"james"@example.com</a>'''
@@ -190,7 +192,7 @@ def test_set_attrs():
     """We can set random attributes on links."""
 
     def set_attr(attrs, new=False):
-        attrs[(None, u'rev')] = u'canonical'
+        attrs[(None, 'rev')] = 'canonical'
         return attrs
 
     assert (
@@ -562,14 +564,14 @@ def test_drop_link_tags():
 
 
 @pytest.mark.parametrize('text, expected', [
-    (u'&lt;br&gt;', u'&lt;br&gt;'),
+    ('&lt;br&gt;', '&lt;br&gt;'),
     (
-        u'&lt;br&gt; http://example.com',
-        u'&lt;br&gt; <a href="http://example.com" rel="nofollow">http://example.com</a>'
+        '&lt;br&gt; http://example.com',
+        '&lt;br&gt; <a href="http://example.com" rel="nofollow">http://example.com</a>'
     ),
     (
-        u'&lt;br&gt; <br> http://example.com',
-        u'&lt;br&gt; <br> <a href="http://example.com" rel="nofollow">http://example.com</a>'
+        '&lt;br&gt; <br> http://example.com',
+        '&lt;br&gt; <br> <a href="http://example.com" rel="nofollow">http://example.com</a>'
     )
 ])
 def test_naughty_unescaping(text, expected):
@@ -625,6 +627,23 @@ def test_email_re_arg():
     )
 
 
+def test_recognized_tags_arg():
+    """Verifies that recognized_tags works"""
+    # The html parser doesn't recognize "sarcasm" as a tag, so it escapes it
+    linker = Linker(recognized_tags=['p'])
+    assert (
+        linker.linkify('<p>http://example.com/</p><sarcasm>') ==
+        '<p><a href="http://example.com/" rel="nofollow">http://example.com/</a></p>&lt;sarcasm&gt;'  # noqa
+    )
+
+    # The html parser recognizes "sarcasm" as a tag and fixes it
+    linker = Linker(recognized_tags=['p', 'sarcasm'])
+    assert (
+        linker.linkify('<p>http://example.com/</p><sarcasm>') ==
+        '<p><a href="http://example.com/" rel="nofollow">http://example.com/</a></p><sarcasm></sarcasm>'  # noqa
+    )
+
+
 def test_linkify_idempotent():
     dirty = '<span>invalid & </span> < extra http://link.com<em>'
     assert linkify(linkify(dirty)) == linkify(dirty)
@@ -666,6 +685,10 @@ class TestLinkify:
         'http://example.com?b=1&c=2',
         '<a href="http://example.com?b=1&amp;c=2">http://example.com?b=1&amp;c=2</a>'
     ),
+    (
+        'http://example.com?b=1&amp;c=2',
+        '<a href="http://example.com?b=1&amp;c=2">http://example.com?b=1&amp;c=2</a>'
+    ),
     (
         'link: https://example.com/watch#anchor',
         'link: <a href="https://example.com/watch#anchor">https://example.com/watch#anchor</a>'
